/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../source/mainwindow.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindow",
        "sigquitApp",
        "",
        "sigRenameFile",
        "slotHandleArguments",
        "listParam",
        "MainWindow::ArgumentType",
        "eType",
        "slotTitleBtnClicked",
        "slotChoosefiles",
        "slotDragSelectedFiles",
        "listFiles",
        "slotCompressLevelChanged",
        "bRootIndex",
        "slotCompressNext",
        "slotCompress",
        "QVariant",
        "val",
        "slotJobFinished",
        "ArchiveJob::JobType",
        "eJobType",
        "PluginFinishType",
        "eFinishType",
        "ErrorType",
        "eErrorType",
        "slotUncompressClicked",
        "strUncompressPath",
        "slotReceiveProgress",
        "dPercentage",
        "slotReceiveCurFileName",
        "strName",
        "slotReceiveFileWriteErrorName",
        "slotQuery",
        "Query*",
        "query",
        "slotExtract2Path",
        "QList<FileEntry>",
        "listSelEntry",
        "ExtractionOptions",
        "stOptions",
        "slotDelFiles",
        "qTotalSize",
        "slotRenameFile",
        "FileEntry",
        "SelEntry",
        "slotReceiveCurArchiveName",
        "strArchiveName",
        "slotOpenFile",
        "entry",
        "strProgram",
        "slotOpenFileChanged",
        "strPath",
        "slotPause",
        "slotContinue",
        "slotCancel",
        "slotAddFiles",
        "strPassword",
        "slotSuccessView",
        "slotSuccessReturn",
        "slotFailureRetry",
        "slotFailureReturn",
        "slotTitleCommentButtonPressed",
        "slotThemeChanged",
        "slotShowShortcutTip",
        "slotFinishCalculateSize",
        "size",
        "strArchiveFullPath",
        "listAddEntry",
        "CompressOptions",
        "listAllEntry",
        "slotCheckFinished",
        "strError",
        "strToolTip"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sigquitApp'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sigRenameFile'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHandleArguments'
        QtMocHelpers::SlotData<void(const QStringList &, MainWindow::ArgumentType)>(4, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotTitleBtnClicked'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotChoosefiles'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDragSelectedFiles'
        QtMocHelpers::SlotData<void(const QStringList &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 11 },
        }}),
        // Slot 'slotCompressLevelChanged'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'slotCompressNext'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCompress'
        QtMocHelpers::SlotData<void(const QVariant &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotJobFinished'
        QtMocHelpers::SlotData<void(ArchiveJob::JobType, PluginFinishType, ErrorType)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { 0x80000000 | 21, 22 }, { 0x80000000 | 23, 24 },
        }}),
        // Slot 'slotUncompressClicked'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'slotReceiveProgress'
        QtMocHelpers::SlotData<void(double)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 28 },
        }}),
        // Slot 'slotReceiveCurFileName'
        QtMocHelpers::SlotData<void(const QString &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
        // Slot 'slotReceiveFileWriteErrorName'
        QtMocHelpers::SlotData<void(const QString &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
        // Slot 'slotQuery'
        QtMocHelpers::SlotData<void(Query *)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'slotExtract2Path'
        QtMocHelpers::SlotData<void(const QList<FileEntry> &, const ExtractionOptions &)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 }, { 0x80000000 | 38, 39 },
        }}),
        // Slot 'slotDelFiles'
        QtMocHelpers::SlotData<void(const QList<FileEntry> &, qint64)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 }, { QMetaType::LongLong, 41 },
        }}),
        // Slot 'slotRenameFile'
        QtMocHelpers::SlotData<void(const FileEntry &, qint64)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 44 }, { QMetaType::LongLong, 41 },
        }}),
        // Slot 'slotReceiveCurArchiveName'
        QtMocHelpers::SlotData<void(const QString &)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 46 },
        }}),
        // Slot 'slotOpenFile'
        QtMocHelpers::SlotData<void(const FileEntry &, const QString &)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 48 }, { QMetaType::QString, 49 },
        }}),
        // Slot 'slotOpenFile'
        QtMocHelpers::SlotData<void(const FileEntry &)>(47, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 43, 48 },
        }}),
        // Slot 'slotOpenFileChanged'
        QtMocHelpers::SlotData<void(const QString &)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 51 },
        }}),
        // Slot 'slotPause'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotContinue'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCancel'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddFiles'
        QtMocHelpers::SlotData<void(const QStringList &, const QString &)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 11 }, { QMetaType::QString, 56 },
        }}),
        // Slot 'slotSuccessView'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSuccessReturn'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFailureRetry'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFailureReturn'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTitleCommentButtonPressed'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotThemeChanged'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowShortcutTip'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFinishCalculateSize'
        QtMocHelpers::SlotData<void(qint64, QString, QList<FileEntry>, CompressOptions, QList<FileEntry>)>(64, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 65 }, { QMetaType::QString, 66 }, { 0x80000000 | 36, 67 }, { 0x80000000 | 68, 39 },
            { 0x80000000 | 36, 69 },
        }}),
        // Slot 'slotCheckFinished'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(70, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 71 }, { QMetaType::QString, 72 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<DMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MainWindowE_t>.metaTypes,
    nullptr
} };

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigquitApp(); break;
        case 1: _t->sigRenameFile(); break;
        case 2: _t->slotHandleArguments((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MainWindow::ArgumentType>>(_a[2]))); break;
        case 3: _t->slotTitleBtnClicked(); break;
        case 4: _t->slotChoosefiles(); break;
        case 5: _t->slotDragSelectedFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 6: _t->slotCompressLevelChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->slotCompressNext(); break;
        case 8: _t->slotCompress((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 9: _t->slotJobFinished((*reinterpret_cast<std::add_pointer_t<ArchiveJob::JobType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<PluginFinishType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<ErrorType>>(_a[3]))); break;
        case 10: _t->slotUncompressClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->slotReceiveProgress((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->slotReceiveCurFileName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->slotReceiveFileWriteErrorName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->slotQuery((*reinterpret_cast<std::add_pointer_t<Query*>>(_a[1]))); break;
        case 15: _t->slotExtract2Path((*reinterpret_cast<std::add_pointer_t<QList<FileEntry>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ExtractionOptions>>(_a[2]))); break;
        case 16: _t->slotDelFiles((*reinterpret_cast<std::add_pointer_t<QList<FileEntry>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 17: _t->slotRenameFile((*reinterpret_cast<std::add_pointer_t<FileEntry>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 18: _t->slotReceiveCurArchiveName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 19: _t->slotOpenFile((*reinterpret_cast<std::add_pointer_t<FileEntry>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 20: _t->slotOpenFile((*reinterpret_cast<std::add_pointer_t<FileEntry>>(_a[1]))); break;
        case 21: _t->slotOpenFileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->slotPause(); break;
        case 23: _t->slotContinue(); break;
        case 24: _t->slotCancel(); break;
        case 25: _t->slotAddFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 26: _t->slotSuccessView(); break;
        case 27: _t->slotSuccessReturn(); break;
        case 28: _t->slotFailureRetry(); break;
        case 29: _t->slotFailureReturn(); break;
        case 30: _t->slotTitleCommentButtonPressed(); break;
        case 31: _t->slotThemeChanged(); break;
        case 32: _t->slotShowShortcutTip(); break;
        case 33: _t->slotFinishCalculateSize((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<FileEntry>>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<CompressOptions>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QList<FileEntry>>>(_a[5]))); break;
        case 34: _t->slotCheckFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PluginFinishType >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Query* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ExtractionOptions >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<FileEntry> >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<FileEntry> >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< FileEntry >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< FileEntry >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< FileEntry >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 3:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CompressOptions >(); break;
            case 4:
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<FileEntry> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::sigquitApp, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::sigRenameFile, 1))
            return;
    }
}

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return DMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::sigquitApp()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MainWindow::sigRenameFile()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN11TitleWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto TitleWidget::qt_create_metaobjectdata<qt_meta_tag_ZN11TitleWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TitleWidget",
        "sigTitleClicked",
        "",
        "sigCommentClicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sigTitleClicked'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sigCommentClicked'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TitleWidget, qt_meta_tag_ZN11TitleWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TitleWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TitleWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TitleWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11TitleWidgetE_t>.metaTypes,
    nullptr
} };

void TitleWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TitleWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigTitleClicked(); break;
        case 1: _t->sigCommentClicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TitleWidget::*)()>(_a, &TitleWidget::sigTitleClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TitleWidget::*)()>(_a, &TitleWidget::sigCommentClicked, 1))
            return;
    }
}

const QMetaObject *TitleWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TitleWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TitleWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TitleWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void TitleWidget::sigTitleClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TitleWidget::sigCommentClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
