/****************************************************************************
** Meta object code from reading C++ file 'uncompressview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../source/tree/uncompressview.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'uncompressview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14UnCompressViewE_t {};
} // unnamed namespace

template <> constexpr inline auto UnCompressView::qt_create_metaobjectdata<qt_meta_tag_ZN14UnCompressViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "UnCompressView",
        "signalExtract2Path",
        "",
        "QList<FileEntry>",
        "listSelEntry",
        "ExtractionOptions",
        "stOptions",
        "signalDelFiles",
        "listCurEntry",
        "qTotalSize",
        "signalRenameFile",
        "FileEntry",
        "SelEntry",
        "signalOpenFile",
        "entry",
        "strProgram",
        "signalAddFiles2Archive",
        "listFiles",
        "strPassword",
        "sigRenameFile",
        "slotPreClicked",
        "slotDragFiles",
        "slotShowRightMenu",
        "QPoint",
        "pos",
        "slotExtract",
        "slotExtract2Here",
        "slotDeleteFile",
        "slotRenameFile",
        "slotOpen",
        "slotOpenStyleClicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalExtract2Path'
        QtMocHelpers::SignalData<void(const QList<FileEntry> &, const ExtractionOptions &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'signalDelFiles'
        QtMocHelpers::SignalData<void(const QList<FileEntry> &, qint64)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 8 }, { QMetaType::LongLong, 9 },
        }}),
        // Signal 'signalRenameFile'
        QtMocHelpers::SignalData<void(const FileEntry &, qint64)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::LongLong, 9 },
        }}),
        // Signal 'signalOpenFile'
        QtMocHelpers::SignalData<void(const FileEntry &, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 14 }, { QMetaType::QString, 15 },
        }}),
        // Signal 'signalOpenFile'
        QtMocHelpers::SignalData<void(const FileEntry &)>(13, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 11, 14 },
        }}),
        // Signal 'signalAddFiles2Archive'
        QtMocHelpers::SignalData<void(const QStringList &, const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 17 }, { QMetaType::QString, 18 },
        }}),
        // Signal 'sigRenameFile'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPreClicked'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDragFiles'
        QtMocHelpers::SlotData<void(const QStringList &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 17 },
        }}),
        // Slot 'slotShowRightMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'slotExtract'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotExtract2Here'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDeleteFile'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRenameFile'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpen'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenStyleClicked'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UnCompressView, qt_meta_tag_ZN14UnCompressViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject UnCompressView::staticMetaObject = { {
    QMetaObject::SuperData::link<DataTreeView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14UnCompressViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14UnCompressViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14UnCompressViewE_t>.metaTypes,
    nullptr
} };

void UnCompressView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UnCompressView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalExtract2Path((*reinterpret_cast<std::add_pointer_t<QList<FileEntry>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ExtractionOptions>>(_a[2]))); break;
        case 1: _t->signalDelFiles((*reinterpret_cast<std::add_pointer_t<QList<FileEntry>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 2: _t->signalRenameFile((*reinterpret_cast<std::add_pointer_t<FileEntry>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 3: _t->signalOpenFile((*reinterpret_cast<std::add_pointer_t<FileEntry>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->signalOpenFile((*reinterpret_cast<std::add_pointer_t<FileEntry>>(_a[1]))); break;
        case 5: _t->signalAddFiles2Archive((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->sigRenameFile(); break;
        case 7: _t->slotPreClicked(); break;
        case 8: _t->slotDragFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 9: _t->slotShowRightMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 10: _t->slotExtract(); break;
        case 11: _t->slotExtract2Here(); break;
        case 12: _t->slotDeleteFile(); break;
        case 13: _t->slotRenameFile(); break;
        case 14: _t->slotOpen(); break;
        case 15: _t->slotOpenStyleClicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ExtractionOptions >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<FileEntry> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<FileEntry> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< FileEntry >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< FileEntry >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< FileEntry >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UnCompressView::*)(const QList<FileEntry> & , const ExtractionOptions & )>(_a, &UnCompressView::signalExtract2Path, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnCompressView::*)(const QList<FileEntry> & , qint64 )>(_a, &UnCompressView::signalDelFiles, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnCompressView::*)(const FileEntry & , qint64 )>(_a, &UnCompressView::signalRenameFile, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnCompressView::*)(const FileEntry & , const QString & )>(_a, &UnCompressView::signalOpenFile, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnCompressView::*)(const QStringList & , const QString & )>(_a, &UnCompressView::signalAddFiles2Archive, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnCompressView::*)()>(_a, &UnCompressView::sigRenameFile, 6))
            return;
    }
}

const QMetaObject *UnCompressView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UnCompressView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14UnCompressViewE_t>.strings))
        return static_cast<void*>(this);
    return DataTreeView::qt_metacast(_clname);
}

int UnCompressView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DataTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void UnCompressView::signalExtract2Path(const QList<FileEntry> & _t1, const ExtractionOptions & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void UnCompressView::signalDelFiles(const QList<FileEntry> & _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void UnCompressView::signalRenameFile(const FileEntry & _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void UnCompressView::signalOpenFile(const FileEntry & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 5
void UnCompressView::signalAddFiles2Archive(const QStringList & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void UnCompressView::sigRenameFile()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
