/****************************************************************************
** Meta object code from reading C++ file 'singlejob.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../source/archivemanager/singlejob.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'singlejob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15SingleJobThreadE_t {};
} // unnamed namespace

template <> constexpr inline auto SingleJobThread::qt_create_metaobjectdata<qt_meta_tag_ZN15SingleJobThreadE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SingleJobThread"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SingleJobThread, qt_meta_tag_ZN15SingleJobThreadE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SingleJobThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SingleJobThreadE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SingleJobThreadE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15SingleJobThreadE_t>.metaTypes,
    nullptr
} };

void SingleJobThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SingleJobThread *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *SingleJobThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SingleJobThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SingleJobThreadE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int SingleJobThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN9SingleJobE_t {};
} // unnamed namespace

template <> constexpr inline auto SingleJob::qt_create_metaobjectdata<qt_meta_tag_ZN9SingleJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SingleJob",
        "slotFinished",
        "",
        "PluginFinishType",
        "eType"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotFinished'
        QtMocHelpers::SlotData<void(PluginFinishType)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SingleJob, qt_meta_tag_ZN9SingleJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SingleJob::staticMetaObject = { {
    QMetaObject::SuperData::link<ArchiveJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9SingleJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9SingleJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9SingleJobE_t>.metaTypes,
    nullptr
} };

void SingleJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SingleJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotFinished((*reinterpret_cast<std::add_pointer_t<PluginFinishType>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PluginFinishType >(); break;
            }
            break;
        }
    }
}

const QMetaObject *SingleJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SingleJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9SingleJobE_t>.strings))
        return static_cast<void*>(this);
    return ArchiveJob::qt_metacast(_clname);
}

int SingleJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ArchiveJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN7LoadJobE_t {};
} // unnamed namespace

template <> constexpr inline auto LoadJob::qt_create_metaobjectdata<qt_meta_tag_ZN7LoadJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LoadJob"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LoadJob, qt_meta_tag_ZN7LoadJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LoadJob::staticMetaObject = { {
    QMetaObject::SuperData::link<SingleJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7LoadJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7LoadJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7LoadJobE_t>.metaTypes,
    nullptr
} };

void LoadJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LoadJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *LoadJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoadJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7LoadJobE_t>.strings))
        return static_cast<void*>(this);
    return SingleJob::qt_metacast(_clname);
}

int LoadJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SingleJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6AddJobE_t {};
} // unnamed namespace

template <> constexpr inline auto AddJob::qt_create_metaobjectdata<qt_meta_tag_ZN6AddJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AddJob"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AddJob, qt_meta_tag_ZN6AddJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AddJob::staticMetaObject = { {
    QMetaObject::SuperData::link<SingleJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6AddJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6AddJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6AddJobE_t>.metaTypes,
    nullptr
} };

void AddJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AddJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *AddJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AddJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6AddJobE_t>.strings))
        return static_cast<void*>(this);
    return SingleJob::qt_metacast(_clname);
}

int AddJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SingleJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN9CreateJobE_t {};
} // unnamed namespace

template <> constexpr inline auto CreateJob::qt_create_metaobjectdata<qt_meta_tag_ZN9CreateJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CreateJob"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CreateJob, qt_meta_tag_ZN9CreateJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CreateJob::staticMetaObject = { {
    QMetaObject::SuperData::link<SingleJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9CreateJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9CreateJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9CreateJobE_t>.metaTypes,
    nullptr
} };

void CreateJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CreateJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CreateJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CreateJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9CreateJobE_t>.strings))
        return static_cast<void*>(this);
    return SingleJob::qt_metacast(_clname);
}

int CreateJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SingleJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN10ExtractJobE_t {};
} // unnamed namespace

template <> constexpr inline auto ExtractJob::qt_create_metaobjectdata<qt_meta_tag_ZN10ExtractJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ExtractJob"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExtractJob, qt_meta_tag_ZN10ExtractJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ExtractJob::staticMetaObject = { {
    QMetaObject::SuperData::link<SingleJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ExtractJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ExtractJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10ExtractJobE_t>.metaTypes,
    nullptr
} };

void ExtractJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExtractJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *ExtractJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExtractJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ExtractJobE_t>.strings))
        return static_cast<void*>(this);
    return SingleJob::qt_metacast(_clname);
}

int ExtractJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SingleJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN9DeleteJobE_t {};
} // unnamed namespace

template <> constexpr inline auto DeleteJob::qt_create_metaobjectdata<qt_meta_tag_ZN9DeleteJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeleteJob"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DeleteJob, qt_meta_tag_ZN9DeleteJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DeleteJob::staticMetaObject = { {
    QMetaObject::SuperData::link<SingleJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9DeleteJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9DeleteJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9DeleteJobE_t>.metaTypes,
    nullptr
} };

void DeleteJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeleteJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *DeleteJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeleteJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9DeleteJobE_t>.strings))
        return static_cast<void*>(this);
    return SingleJob::qt_metacast(_clname);
}

int DeleteJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SingleJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN9RenameJobE_t {};
} // unnamed namespace

template <> constexpr inline auto RenameJob::qt_create_metaobjectdata<qt_meta_tag_ZN9RenameJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RenameJob"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RenameJob, qt_meta_tag_ZN9RenameJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RenameJob::staticMetaObject = { {
    QMetaObject::SuperData::link<SingleJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9RenameJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9RenameJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9RenameJobE_t>.metaTypes,
    nullptr
} };

void RenameJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RenameJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *RenameJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RenameJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9RenameJobE_t>.strings))
        return static_cast<void*>(this);
    return SingleJob::qt_metacast(_clname);
}

int RenameJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SingleJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN7OpenJobE_t {};
} // unnamed namespace

template <> constexpr inline auto OpenJob::qt_create_metaobjectdata<qt_meta_tag_ZN7OpenJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OpenJob",
        "slotFinished",
        "",
        "PluginFinishType",
        "eType"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotFinished'
        QtMocHelpers::SlotData<void(PluginFinishType)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OpenJob, qt_meta_tag_ZN7OpenJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OpenJob::staticMetaObject = { {
    QMetaObject::SuperData::link<SingleJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7OpenJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7OpenJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7OpenJobE_t>.metaTypes,
    nullptr
} };

void OpenJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OpenJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotFinished((*reinterpret_cast<std::add_pointer_t<PluginFinishType>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PluginFinishType >(); break;
            }
            break;
        }
    }
}

const QMetaObject *OpenJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OpenJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7OpenJobE_t>.strings))
        return static_cast<void*>(this);
    return SingleJob::qt_metacast(_clname);
}

int OpenJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SingleJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN9UpdateJobE_t {};
} // unnamed namespace

template <> constexpr inline auto UpdateJob::qt_create_metaobjectdata<qt_meta_tag_ZN9UpdateJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "UpdateJob"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UpdateJob, qt_meta_tag_ZN9UpdateJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject UpdateJob::staticMetaObject = { {
    QMetaObject::SuperData::link<SingleJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9UpdateJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9UpdateJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9UpdateJobE_t>.metaTypes,
    nullptr
} };

void UpdateJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UpdateJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *UpdateJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UpdateJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9UpdateJobE_t>.strings))
        return static_cast<void*>(this);
    return SingleJob::qt_metacast(_clname);
}

int UpdateJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SingleJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN10CommentJobE_t {};
} // unnamed namespace

template <> constexpr inline auto CommentJob::qt_create_metaobjectdata<qt_meta_tag_ZN10CommentJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CommentJob"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CommentJob, qt_meta_tag_ZN10CommentJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CommentJob::staticMetaObject = { {
    QMetaObject::SuperData::link<SingleJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CommentJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CommentJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10CommentJobE_t>.metaTypes,
    nullptr
} };

void CommentJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CommentJob *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CommentJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CommentJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CommentJobE_t>.strings))
        return static_cast<void*>(this);
    return SingleJob::qt_metacast(_clname);
}

int CommentJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SingleJob::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN10ComplexJobE_t {};
} // unnamed namespace

template <> constexpr inline auto ComplexJob::qt_create_metaobjectdata<qt_meta_tag_ZN10ComplexJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ComplexJob",
        "slotHandleSingleJobProgress",
        "",
        "dPercentage",
        "slotHandleSingleJobCurFileName",
        "strName",
        "slotHandleExtractFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotHandleSingleJobProgress'
        QtMocHelpers::SlotData<void(double)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'slotHandleSingleJobCurFileName'
        QtMocHelpers::SlotData<void(const QString &)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'slotHandleExtractFinished'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ComplexJob, qt_meta_tag_ZN10ComplexJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ComplexJob::staticMetaObject = { {
    QMetaObject::SuperData::link<ArchiveJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ComplexJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ComplexJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10ComplexJobE_t>.metaTypes,
    nullptr
} };

void ComplexJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ComplexJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotHandleSingleJobProgress((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->slotHandleSingleJobCurFileName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->slotHandleExtractFinished(); break;
        default: ;
        }
    }
}

const QMetaObject *ComplexJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ComplexJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ComplexJobE_t>.strings))
        return static_cast<void*>(this);
    return ArchiveJob::qt_metacast(_clname);
}

int ComplexJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ArchiveJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN10ConvertJobE_t {};
} // unnamed namespace

template <> constexpr inline auto ConvertJob::qt_create_metaobjectdata<qt_meta_tag_ZN10ConvertJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ConvertJob",
        "slotHandleExtractFinished",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotHandleExtractFinished'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConvertJob, qt_meta_tag_ZN10ConvertJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ConvertJob::staticMetaObject = { {
    QMetaObject::SuperData::link<ComplexJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ConvertJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ConvertJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10ConvertJobE_t>.metaTypes,
    nullptr
} };

void ConvertJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConvertJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotHandleExtractFinished(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *ConvertJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ConvertJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ConvertJobE_t>.strings))
        return static_cast<void*>(this);
    return ComplexJob::qt_metacast(_clname);
}

int ConvertJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ComplexJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN14StepExtractJobE_t {};
} // unnamed namespace

template <> constexpr inline auto StepExtractJob::qt_create_metaobjectdata<qt_meta_tag_ZN14StepExtractJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StepExtractJob",
        "slotHandleExtractFinished",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotHandleExtractFinished'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StepExtractJob, qt_meta_tag_ZN14StepExtractJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StepExtractJob::staticMetaObject = { {
    QMetaObject::SuperData::link<ComplexJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14StepExtractJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14StepExtractJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14StepExtractJobE_t>.metaTypes,
    nullptr
} };

void StepExtractJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StepExtractJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotHandleExtractFinished(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *StepExtractJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StepExtractJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14StepExtractJobE_t>.strings))
        return static_cast<void*>(this);
    return ComplexJob::qt_metacast(_clname);
}

int StepExtractJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ComplexJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
