// Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef MIMETYPES_H
#define MIMETYPES_H

#include "commonstruct.h"

CustomMimeType determineMimeType(const QString &filename);

#endif // MIMETYPES_H
